package net.sjava.cassandra.manager;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

import net.sjava.cassandra.model.Category;
import net.sjava.cassandra.util.ColumnUtil;
import net.sjava.cassandra.util.ConnectionUtil;
import net.sjava.cassandra.util.ConverterUtil;

import org.apache.cassandra.thrift.Cassandra;
import org.apache.cassandra.thrift.Column;
import org.apache.cassandra.thrift.ColumnOrSuperColumn;
import org.apache.cassandra.thrift.ColumnParent;
import org.apache.cassandra.thrift.ColumnPath;
import org.apache.cassandra.thrift.ConsistencyLevel;
import org.apache.cassandra.thrift.KeyRange;
import org.apache.cassandra.thrift.KeySlice;
import org.apache.cassandra.thrift.SlicePredicate;
//import org.apache.log4j.Logger;


/**
 * α īװ ϴ Ŵ Ŭ 
 * 
 * @author mcsong@gmail.com
 *
 */
public class CategoryManager {

	//private static final Logger LOG = Logger.getLogger(CategoryManager.class);
	private static String UTF8 ="UTF8";
	
	private Cassandra.Client client;
	private String cfName = "category";
	private ColumnParent cp = new ColumnParent(cfName);	
	
	// ̱ ü
	private static CategoryManager instance = new CategoryManager();
 
	private CategoryManager() { 
		try {
			client  = ConnectionUtil.createConnection("BLOG");
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public static CategoryManager instance() {
		return instance;
	}
	
	/**
	 * īװ ߰Ѵ. 
	 * @param key
	 * @param id
	 * @param name
	 * @throws Exception
	 */
	public void addCategory(String key, String id, String name) throws Exception  {
		//LOG.debug(String.format("addCategory(%s, %s, %s) start", key, id, name));
		
		long timestamp = System.currentTimeMillis();
		
		Column idColumn = ColumnUtil.createColumn("category_id".getBytes(UTF8), id.getBytes(UTF8), timestamp);
    	Column nameColumn = ColumnUtil.createColumn("category_name".getBytes(UTF8), name.getBytes(UTF8), timestamp);
    	
    	client.insert(ConverterUtil.stringToByteBuffer(key, UTF8), cp, idColumn, ConsistencyLevel.ONE);
    	client.insert(ConverterUtil.stringToByteBuffer(key, UTF8), cp, nameColumn, ConsistencyLevel.ONE);
    	
		//LOG.debug(String.format("addCategory(%s, %s, %s) end", key, id, name));
	}
		
	/**
	 * īװ Ʈ ´. 
	 */
	public List<Category> getCategoryies() throws Exception {
		SlicePredicate predicate = new SlicePredicate();
		predicate.addToColumn_names(ByteBuffer.wrap("category_id".getBytes("UTF8")));
		predicate.addToColumn_names(ByteBuffer.wrap("category_name".getBytes("UTF8")));
		
		KeyRange keyRange = new KeyRange();
		keyRange.start_key = ByteBuffer.allocate(0);
		keyRange.end_key = ByteBuffer.allocate(0);
		
		List<Category> categorise = new ArrayList<Category>();
		
		// Ű ̽ ȯȴ
		List<KeySlice> results = client.get_range_slices(cp, predicate, keyRange, ConsistencyLevel.ONE);
		
		for (KeySlice keySlice : results) {
			List<ColumnOrSuperColumn> cosc = keySlice.getColumns();
			Category category = new Category();
			category.setKey(new String(keySlice.getKey()));
			
			String name = null;			
			for (int i = 0; i < cosc.size(); i++) {
				Column c = cosc.get(i).getColumn();
				name = ConverterUtil.byteBufferToString(c.name);
				 
				if("category_id".equals(name))
					category.setId(ConverterUtil.byteBufferToString(c.value));
				
				if("category_name".equals(name))
					category.setName(ConverterUtil.byteBufferToString(c.value));
			}
			categorise.add(category);
		}
		
		return categorise;
	}
	
	/**
	 * οŰ شϴ ÷ Ѵ.  
	 * 
	 * @param key
	 * @throws Exception
	 */
	public void deleteCategory(String key) throws Exception {
		long timestamp = System.currentTimeMillis();
		
		ColumnPath cPath = new ColumnPath();
		cPath.setColumn_family(cfName);
		
		client.remove(ConverterUtil.stringToByteBuffer(key, UTF8), cPath, timestamp, ConsistencyLevel.ONE);
	}
	
}
